#define DE_Quaternion
#info  DE Quaternion
#info   float qtDE(vec3 p)

// DE implementation based on Syntopia + Subblue + Keenan Crane
// http://www.subblue.com/blog/2009/9/20/quaternion_julia
// Added box folding, sphere folding and iterative rotation

#group FQT
uniform vec3 qtPTRotate; slider[(0,0,0),(0,0,0),(360,360,360)];
uniform vec3 qtPTTranslate; slider[(-50,-50,-50),(0,0,0),(50,50,50)];
uniform float qtPTScale; slider[0.0,1,5.10]
//uniform vec4 qtScale; slider[(0.,0.,0.,0.),(0.5,0.5,0.5,0.5),(1.,1.,1.,1.)]

uniform int qtIterations;  slider[0,16,100]
uniform float qtThreshold; slider[0,10,100]
uniform vec4 qtC; slider[(-1,-1,-1,-2),(0.18,0.88,0.24,0.16),(1,1,1,2)]
uniform vec3 qtRotVector; slider[(0,0,0),(0,0,1),(1,1,1)]
uniform float qtRotAngle; slider[0.00,0,180]
uniform int foldBoxStartIt; slider[0,0,256]
uniform int foldBoxIts; slider[0,0,16]
uniform vec3 foldBoxLimit; slider[(-5,-5,-5),(0,0,0),(5,5,5)]
uniform vec3 foldBoxValue; slider[(-5,-5,-5),(2,2,2),(5,5,5)]
uniform vec3 foldBoxOffset; slider[(-5,-5,-5),(0.,0.,0.),(5,5,5)]

uniform int foldSphereStartIt; slider[0,0,256]
uniform int foldSphereIts; slider[0,0,16]
uniform float foldSphereMinRad;  slider[0,0.25,2.0]

mat3 qtrot = rotationMatrix3(normalize(qtRotVector), qtRotAngle);

float qtDE(vec3 pos) {
	
	vec4 dp = vec4(1.0, 0.0,0.0,0.0);
	vec4 p = vec4(pos, 0.0);
	
	for (int i = 0; i <qtIterations; i++){
		
		// base
		dp = 2.0 * qMult(p,dp);
		p = qSq(p) + qtC;
		
		// rotation
		p.xyz *= qtrot;
		
		// scaling, experimental, generates artifacts
		//p = qMult(p, qtScale);		
		//p = qMult(p, normalize(qtScale));		
		
		// Box Folding (v2) - separate axes + value + offset
		if (foldBoxIts > 0){
			if ( i >= foldBoxStartIt && i < foldBoxStartIt + foldBoxIts){
				p.xyz = clamp(p.xyz, -foldBoxLimit, foldBoxLimit) * foldBoxValue - p.xyz + foldBoxOffset;
			}
		}
		
		// Sphere folding
		if(foldSphereIts > 0){
			if ( i >= foldSphereStartIt && i < foldSphereStartIt + foldSphereIts){
				float r2 = dot(p.xyz, p.xyz);
				p *= clamp(max(foldSphereMinRad/r2, foldSphereMinRad), 0.0, 1.0);
			}
		}
		
		float p2 = dot(p,p);
		orbitTrap = min(orbitTrap, abs(vec4(p.xyz,p2)));
		if (p2 > qtThreshold) break;
	}
	
	// DE
	float r = length(p);
	return  0.5 * r * log(r) / length(dp);
}

