#donotrun

#group Transform
// Added global transformations to the space

// Translate
uniform vec3 sTranslate; slider[(-50,-50,-50),(0,0,0),(50,50,50)]
// Rotate
uniform vec3 sRotate; slider[(-180,-180,-180),(0,0,0),(180,180,180)]
// Scale
//uniform vec3 sScale; slider[(0.001,0.001,0.001),(1,1,1),(10,10,10)]
uniform float sScale; slider[0.1,1.,10.]
// Twist
uniform float sTwist; slider[-2.0,0.0,2.0]
// Stereographic projection

#group Clipping
// Allows for slicing the fractal and rendering only a box-limited volume
// Box may be sclaed/rotated/translated

// Clip scene contents to box
uniform bool clipToBox; checkbox[false] NotLocked
// Use that to see the box
uniform bool showClippingBox; checkbox[false] NotLocked
// Clipping Box size
uniform vec3 cBoxSize; slider[(0,0,0),(5,5,5),(20,20,20)]
// Clipping Box Translate
uniform vec3 cBoxTranslate; slider[(-50,-50,-50),(0,0,0),(50,50,50)]
// Clipping Box Rotate
uniform vec3 cBoxRotate; slider[(-180,-180,-180),(0,0,0),(180,180,180)]

// -------------------------------- Primitives -------------------------------

// Returns DE to a box
float DEBox(vec3 p, vec3 box){
	vec3 boxd = abs(p) -  box;
	return min(max(boxd.x,max(boxd.y,boxd.z)),0.0) + length(max(boxd,0.0));
}

// Returns DE to a box
float DESphere(vec3 p, vec3 sPos, float r){
	return length(p-sPos)-r;
}

// -------------------------------- Vector Operations -------------------------------

vec3 opSRT(vec3 p, float s, vec3 t, vec3 r){
	if (s==0) return p;
	else return p*rotationMatrixXYZ(r)/s + t;
}

vec3 opTRS(vec3 p, float s, vec3 t, vec3 r){
	if (s==0) return p;
	else return ((p+t)*rotationMatrixXYZ(r)/s)*-1;
}

vec3 opTranslate(vec3 p, vec3 d){
	mat4 m = translate(d);
	vec3 p1 = m * vec4(p.x, p.y, p.z,1);
	return p1;
}

vec3 opRotateVA(vec3 p, vec3 v, float angle){
	if (length(v) > 0.0){
		mat4 m = rotationMatrix(normalize(v), angle);
		vec3 p1 = m * vec4(p.x, p.y, p.z,1.0);
	return p1;
	} else {
		return p;	
	}
}

vec3 opRotateXYZ(vec3 p, vec3 angle){
	mat3 m = rotationMatrixXYZ(angle);
	vec3 p1 = m * vec3(p.x, p.y, p.z);
	return p1;
}

vec3 opScale( vec3 p, float s){
	return s!=0 ? (p/s) : p;
}

vec3 opNUScale( vec3 p, vec3 s){
	return (p / s);
}

vec3 opTwist( vec3 p, float tw){
	if (tw > 0){
		float c = cos(tw*p.y);
		float s = sin(tw*p.y);
		mat2  m = mat2(c,-s,s,c);
		vec3  q = vec3(p.xz*m,p.y);
	return q;
	}else {
		return p;
	}
}

// -------------------------------- Functions -------------------------------

// -------------------------------- Space mixing -------------------------------

// exponential smooth min (k = 32);
float sminexp( float a, float b, float k ){
	float res = exp( -k*a ) + exp( -k*b );
	return -log( res )/k;
}

// polynomial smooth min (k = 0.1);
float sminpol( float a, float b, float k ){
	float h = clamp( 0.5+0.5*(b-a)/k, 0.0, 1.0 );
	return mix( b, a, h ) - k*h*(1.0-h);
}

// power smooth min (k = 8);
float sminpow( float a, float b, float k ){
	a = pow( a, k ); b = pow( b, k );
	return pow( (a*b)/(a+b), 1.0/k );
}	

// hybrid
float hybrid(float de1, float de2, int mode){
	switch (mode){
		case 0: return de1;
		case 1: return de1 + de2;
		case 2: return de1 - de2;
		case 3: return de1 * de2;
		case 4: return de2!=0 ? de1 / de2 : de1;
		case 5: return min(de1, de2);
		case 6: return max(de1, de2);
		case 7: return pow(de1, de2);
	}
}

// Hybrid Smooth
float hybridSmooth(float de1, float de2, int mode, float k){
	switch (mode){
		case 0: return de1;
		case 1: return de1 + de2;
		case 2: return de1 - de2;
		case 3: return de1 * de2;
		case 4: return de2!=0 ? de1 / de2 : de1;
		case 5: return min(de1, de2);
		case 6: return max(de1, de2);
		case 7: return pow(de1, de2);
		case 8: return sminexp(de1, de2, k);
		case 9: return sminpol(de1, de2, k);
	}
}


