// Output generated from file: C:/Users/Rachel/programs/Fragmentarium/Fragmentarium/Examples/pk116/__Scene_All_Features_Example.frag
// Created: Fri 25. Sep 22:05:14 2015
#info Hybrid Fractals - Main Frag
#info Quaternion Julia DE + Box Folding + Sphere Folding
#info Mandelbox, Mandelbulb, Menger Sphere
#info ------

// Flags
#define USE_EIFFIE_SHADOW
#define PERFECT_DE
#define USE_PK_IBL
#define USE_PK_BACKGROUND
#define USE_PK_DIRT
#define USE_CLIPPING		// Enables box culling for the fractal
//#define USE_KN2_SHADER 	// Uses older Kn2 shading, faster, less features
#define USE_IQ_CLOUDS 	// Enables IQ clouds
#define USE_IQ_CLOUDS_MULTILAYER // Enables multilayer clouds
#define USE_KN_VOLUMETRIC	// Enables volumetric effects

//#define TAG_LAYERS 		// Uncomment this to output alpha pass
//#define DEPTH_PASS 		// Uncomment this to output depth pass
// use both in conjunction with #include "DE-PK-Extras-Render-Elements.frag"

// Core
//#include "QS-Camera-1.0.frag"
#include "Math-Utils.frag"
#include "Math-Quaternion.frag"

// Adds transformations of global space
#include "Space-Transforms.frag" 	
#include "Space-Noise.frag"			// Adds spatial noise of global space [slow]
#include "Space-Stereo-Map.frag"	// Adds stereo mapping of global space [slow]
#include "DE-PK-1.1.6.frag"
//#include "DE-PK-Extras-Bump.frag"					// Adds Bump map texture for the main object
//#include "DE-PK-Extras-Render-Elements.frag"	// Adds Layer Tagging & Z-Depth output

#include "Presets-Camera-Z-Up.frag"
#include "Presets-PK-Raytracer-1.1.6.frag"

#include "DE-Quaternion.frag"

// ---------------------------------------- Fractals -------------------------------------------

#group DE
uniform bool 	DE_TestBall; checkbox[false]
uniform float 	DE_TestBallR; slider[0,1,10]


// ---------------------------------------- Init -------------------------------------------
/*
void init() {
}*/

// ----------------------------------------Main -------------------------------------------

float DE(vec3 z) {
	vec3 z0 = z; // Store z 

	// Test Ball
	if(DE_TestBall) return DESphere(z, -sTranslate, DE_TestBallR);
	
	// Global transformations
	z = opTRS(z, sScale, -sTranslate, sRotate);
	
	// Extras
	#ifdef USE_SPACE_STEREOMAP
	z = stereoMap(z);
	#endif

	#ifdef USE_SPACE_NOISE
	z = spaceNoise(z);
	#endif
	
	// Main DE part
	float DE = 0;
	
	vec3 z1 = opTRS(z, qtPTScale, qtPTTranslate, qtPTRotate);
	DE = qtDE(z1);

	#ifdef USE_CLIPPING
	vec3 zb;
	zb = opRotateXYZ(z, cBoxRotate);
	zb = opTranslate(zb, cBoxTranslate - sTranslate);
	float DE_MAX =  DEBox(zb, cBoxSize);
	
	if (clipToBox) DE = max(DE, DE_MAX);
	if (showClippingBox) DE = min(DE, DE_MAX);
	#endif
	
	return DE;
	
}

// ---------------------------------------- Presets -------------------------------------------


#preset default
sTranslate = 0,0,0
sRotate = -22.248,0,-61.6438
sScale = 1
sTwist = 0
clipToBox = false
showClippingBox = false
cBoxSize = 5,5,5
cBoxTranslate = 0,0,0
cBoxRotate = 0,0,0
FOV = 1.35
AngFOV = 115
UseAngFOV = true
UseFocalLength = false
FocalLength = 14
FilmGate = 36
UpLock = true
Eye = 0.0766301,1.29183,-0.123738
Ey_ = 0,0,0
Target = 0.106936,-1.69061,-0.446472
Targe_ = 0,0,0
Up = -0.0213669,-0.107773,0.993946
FocalPlane = 1
Aperture = 0
InFocusAWidth = 0
ApertureNbrSides = 5
ApertureRot = 0
ApStarShaped = false
DryPass = false
Gamma = 2.2
ToneMapping = 3
Exposure = 1
Brightness = 1
Contrast = 1
Saturation = 1
GaussianWeight = 1
AntiAliasScale = 2
Bloom = false
BloomIntensity = 0.25
BloomPow = 2
BloomTaps = 4
Detail = -3.5
DetailAO = -0.5
AutoFocus = false
FudgeFactor = 0.95
MaxRaySteps = 768
MaxDistance = 32
MaxRayStepsDiv = 1.8
Dither = 0.5
NormalBackStep = 1
FloorEnable = false
FloorGrid = false
FloorMatte = false
FloorNormal = 0,0,1
FloorHeight = 0
FloorExcludePost = false
fDiffuse = 0.5,0.5,0.5
fDiffExp = 2
fSpecular = 1,1,1,0.5
fSpecularExp = 4
fSpecularMax = 2
fDirt = 0.5
oDiffuse = 1,1,1
oDiffExp = 2
oSpecular = 1,1,1,0.5
oSpecularExp = 4
oSpecularMax = 2
oDirt = 1
oMatte = false
AO = 0,0,0,1
AOP = 1.5
AoCorrect = 0
SunLight = 1,1,1,0.4
SunDir = 0.1,0.1
SunDir3 = 1,0,1
SunDirUseVec = true
SunMatteShad = 0
DebugSun = false
AmbientLight = 1,1,1,0.75
AmbientLightMin = 0.1
AmbientMatteShad = 0
Glow = 1,1,1,0
GlowMax = 20
Fog = 0.92572
FogExp = 2
Reflection = 1,1,1
ReflectionsNumber = 2
HardShadow = 0.85
ShadowSoft = 10
SpotGlow = false
Spot = 1,1,1,0
SpotPos = 0,0,1.75824
SpotSize = 0
SpotFallOff = 0
SpotGlowRad = 0
SpotGlowExp = 1
SpotMatteShad = 0
OrbitStrength = 0
X = 0.5,0.6,0.6,0.7
Y = 1,0.6,0,0.4
Z = 0.8,0.78,1,0.5
R = 0.4,0.7,1,0.12
CycleColors = false
Cycles = 1.1
HF_Fallof = 0.1
HF_Const = 0
HF_Intensity = 0
HF_Dir = 0,0,1
HF_Offset = 0
HF_Color = 1,1,1,1
HF_Scatter = 0
HF_Anisotropy = 0,0,0
HF_FogIter = 1
HF_CastShadow = false
BgColor = 0.5,0.5,0.5,1
BgGradient = 0.3
BgUseMap = true
BgExcludeFromPost = true
BgMapEquirectangular = false
BgMap = C:/Users/Rachel/programs/Mandelbulb3Dv189x2/M3Maps/3350.jpg
BgMapScale = 1,1,1
BgMapOffset = 0,0,0
EnvDiffUseBackground = false
EnvSpecUseBackground = false
EnvMapSpecular = Ditch-River_2k.hdr
EnvMapDiffuse = Ditch-River_Env.hdr
EnvLightFactor = 1,1,1
EnvDiffussionFade = 0
EnvSpecularFade = 0
EnvDiffussionGain = 1
EnvSpecularGain = 1
EnvDiffuseOffset = 0,0
EnvSpecularOffset = 0,0
dirtAccumulation = 0,0
dirtNoise = 0.5,0.5
dirtColorNormal = 0,0,0
dirtColorGlow = 0,0,0
dirtDirNormal = 0,0,1
dirtDirGlow = 0,0,1
dirtDirectionality = 1,1
dirtFalloff = 1,1
mtime = 0
CloudScale = 1
CloudFlatness = 0
CloudTops = 0
CloudBase = -1
CloudDensity = 0
CloudRoughness = 1
CloudContrast = 1
CloudColor = 0.5,0.5,0.5
SunLightColor = 0.75,0.75,0.75
WindDir = 0,1,0
WindSpeed = 1
HCloudScale = 1
HCloudFlatness = 0.5
HCloudTops = 2
HCloudBase = 1
HCloudDensity = 0
HCloudRoughness = 1
HCloudContrast = 1
HCloudColor = 0.5,0.5,0.5
HCloudWindDir = 0,1,0
HCloudWindSpeed = 1
qtPTRotate = 0,0,0
qtPTTranslate = 0,0,0
qtPTScale = 1
qtIterations = 16
qtThreshold = 10
qtC = 0.18,0.88,0.24,0.16
qtRotVector = 0,0,1
qtRotAngle = 0
foldBoxStartIt = 0
foldBoxIts = 0
foldBoxLimit = 0,0,0
foldBoxValue = 2,2,2
foldBoxOffset = 0,0,0
foldSphereStartIt = 0
foldSphereIts = 0
foldSphereMinRad = 0.25
DE_TestBall = false
DE_TestBallR = 1
#endpreset

